module axi_reader_16
#(
  parameter BUF_WIDTH = 16
)
(
  input  clk,
  input  reset,
  output [16-1 : 0] fifo_dout,
  output  fifo_we,
  output  fifo_wclk,
  input [32-1 : 0] fifo_length,
  input  fifo_full,
  output [32-1 : 0] S_AXI_ARADDR,
  output [8-1 : 0] S_AXI_ARLEN,
  output  S_AXI_ARVALID,
  input  S_AXI_ARREADY,
  output [3-1 : 0] S_AXI_ARSIZE,
  output [2-1 : 0] S_AXI_ARBURST,
  output [4-1 : 0] S_AXI_ARCACHE,
  output [3-1 : 0] S_AXI_ARPROT,
  input [16-1 : 0] S_AXI_RDATA,
  input [2-1 : 0] S_AXI_RRESP,
  input  S_AXI_RLAST,
  input  S_AXI_RVALID,
  output  S_AXI_RREADY,
  input  request,
  output  busy,
  input [32-1 : 0] addr,
  input [8-1 : 0] len
);

  wire  clk_sig;
  wire  reset_sig;
  reg [16-1 : 0] fifo_dout_sig = 0;
  reg  fifo_we_sig = 1'b0;
  wire  fifo_wclk_sig;
  wire [32-1 : 0] fifo_length_sig;
  wire  fifo_full_sig;
  reg [32-1 : 0] S_AXI_ARADDR_sig = 0;
  reg [8-1 : 0] S_AXI_ARLEN_sig = 0;
  reg  S_AXI_ARVALID_sig = 1'b0;
  wire  S_AXI_ARREADY_sig;
  wire [3-1 : 0] S_AXI_ARSIZE_sig;
  wire [2-1 : 0] S_AXI_ARBURST_sig;
  wire [4-1 : 0] S_AXI_ARCACHE_sig;
  wire [3-1 : 0] S_AXI_ARPROT_sig;
  wire [16-1 : 0] S_AXI_RDATA_sig;
  wire [2-1 : 0] S_AXI_RRESP_sig;
  wire  S_AXI_RLAST_sig;
  wire  S_AXI_RVALID_sig;
  reg  S_AXI_RREADY_sig = 1'b0;
  wire  request_sig;
  reg  busy_sig = 1'b0;
  wire [32-1 : 0] addr_sig;
  wire [8-1 : 0] len_sig;

  wire  tmp_0001;
  wire  tmp_0002;
  localparam main_IDLE = 32'd0;
  localparam main_init = 32'd1;
  localparam main_read0 = 32'd2;
  localparam main_read = 32'd3;
  reg [31:0] main = main_IDLE;
  reg [31:0] main_prev = main_IDLE;
  reg signed [32-1 : 0] main_delay = 0;
  wire  tmp_0003;
  wire signed [8-1 : 0] tmp_0004;
  reg signed [8-1 : 0] read_length = 0;
  wire  tmp_0005;
  wire signed [8-1 : 0] tmp_0006;
  wire signed [8-1 : 0] tmp_0007;
  wire signed [8-1 : 0] tmp_0008;
  wire signed [8-1 : 0] tmp_0009;
  wire  tmp_0010;
  wire  tmp_0011;
  wire  tmp_0012;
  wire  tmp_0013;
  wire  tmp_0014;
  wire  tmp_0015;
  wire  tmp_0016;
  wire  tmp_0017;
  wire  tmp_0018;


  assign clk_sig = clk;
  assign reset_sig = reset;
  assign fifo_dout = fifo_dout_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      fifo_dout_sig <= 0;
    end else begin
      if (main == main_read0) begin
        fifo_dout_sig <= S_AXI_RDATA_sig;
      end else if (main == main_read) begin
        fifo_dout_sig <= S_AXI_RDATA_sig;
      end
    end
  end

  assign fifo_we = fifo_we_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      fifo_we_sig <= 1'b0;
    end else begin
      if (main == main_IDLE) begin
        fifo_we_sig <= 1'b0;
      end else if (main == main_read0) begin
        fifo_we_sig <= tmp_0002;
      end else if (main == main_read) begin
        fifo_we_sig <= tmp_0002;
      end
    end
  end

  assign fifo_wclk = fifo_wclk_sig;
  assign fifo_wclk_sig = clk_sig;

  assign fifo_length_sig = fifo_length;
  assign fifo_full_sig = fifo_full;
  assign S_AXI_ARADDR = S_AXI_ARADDR_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      S_AXI_ARADDR_sig <= 0;
    end else begin
      if (main == main_init) begin
        S_AXI_ARADDR_sig <= addr_sig;
      end
    end
  end

  assign S_AXI_ARLEN = S_AXI_ARLEN_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      S_AXI_ARLEN_sig <= 0;
    end else begin
      if (main == main_init) begin
        S_AXI_ARLEN_sig <= tmp_0004;
      end
    end
  end

  assign S_AXI_ARVALID = S_AXI_ARVALID_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      S_AXI_ARVALID_sig <= 1'b0;
    end else begin
      if (main == main_init) begin
        S_AXI_ARVALID_sig <= 1'b1;
      end else if (main == main_read0) begin
        S_AXI_ARVALID_sig <= tmp_0005;
      end
    end
  end

  assign S_AXI_ARREADY_sig = S_AXI_ARREADY;
  assign S_AXI_ARSIZE = S_AXI_ARSIZE_sig;
  assign S_AXI_ARSIZE_sig = 3'b001;

  assign S_AXI_ARBURST = S_AXI_ARBURST_sig;
  assign S_AXI_ARBURST_sig = 2'b01;

  assign S_AXI_ARCACHE = S_AXI_ARCACHE_sig;
  assign S_AXI_ARCACHE_sig = 4'h3;

  assign S_AXI_ARPROT = S_AXI_ARPROT_sig;
  assign S_AXI_ARPROT_sig = 3'b000;

  assign S_AXI_RDATA_sig = S_AXI_RDATA;
  assign S_AXI_RRESP_sig = S_AXI_RRESP;
  assign S_AXI_RLAST_sig = S_AXI_RLAST;
  assign S_AXI_RVALID_sig = S_AXI_RVALID;
  assign S_AXI_RREADY = S_AXI_RREADY_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      S_AXI_RREADY_sig <= 1'b0;
    end else begin
      if (main == main_IDLE) begin
        S_AXI_RREADY_sig <= 1'b1;
      end
    end
  end

  assign request_sig = request;
  assign busy = busy_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      busy_sig <= 1'b0;
    end else begin
      if (main == main_IDLE) begin
        busy_sig <= tmp_0003;
      end
    end
  end

  assign addr_sig = addr;
  assign len_sig = len;

  assign tmp_0001 = S_AXI_ARREADY_sig == 1'b1 ? 1'b1 : 1'b0;
  assign tmp_0002 = S_AXI_RVALID_sig == 1'b1 ? 1'b1 : 1'b0;
  assign tmp_0003 = request_sig == 1'b1 ? 1'b1 : 1'b0;
  assign tmp_0004 = len_sig - 1;
  assign tmp_0005 = tmp_0001 == 1'b1 ? 1'b0 : 1'b1;
  assign tmp_0006 = read_length - 1;
  assign tmp_0007 = tmp_0002 == 1'b1 ? tmp_0006 : read_length;
  assign tmp_0008 = read_length - 1;
  assign tmp_0009 = tmp_0002 == 1'b1 ? tmp_0008 : read_length;
  assign tmp_0010 = fifo_full_sig == 1'b0 ? 1'b1 : 1'b0;
  assign tmp_0011 = S_AXI_RLAST_sig == 1'b1 ? 1'b1 : 1'b0;
  assign tmp_0012 = tmp_0002 & tmp_0011;
  assign tmp_0013 = tmp_0012 == 1'b1 ? 1'b1 : 1'b0;
  assign tmp_0014 = tmp_0002 & tmp_0011;
  assign tmp_0015 = tmp_0014 == 1'b1 ? 1'b1 : 1'b0;
  assign tmp_0016 = read_length == 1 ? 1'b1 : 1'b0;
  assign tmp_0017 = tmp_0002 & tmp_0016;
  assign tmp_0018 = tmp_0017 == 1'b1 ? 1'b1 : 1'b0;

  always @(posedge clk) begin
    if(reset == 1'b1) begin
      main <= main_IDLE;
      main_delay <= 32'h0;
    end else begin
      case (main)
        main_IDLE : begin
          if (tmp_0003 == 1'b1) begin
            main <= main_init;
          end
        end
        main_init : begin
          if (tmp_0010 == 1'b1) begin
            main <= main_read0;
          end
        end
        main_read0 : begin
          if (tmp_0001 == 1'b1) begin
            main <= main_read;
          end
        end
        main_read : begin
          if (tmp_0013 == 1'b1) begin
            main <= main_IDLE;
          end else if (tmp_0015 == 1'b1) begin
            main <= main_IDLE;
          end else if (tmp_0018 == 1'b1) begin
            main <= main_IDLE;
          end
        end
      endcase
    end
  end


  always @(posedge clk) begin
    if(reset == 1'b1) begin
      read_length <= 0;
    end else begin
      if (main == main_init) begin
        read_length <= len_sig;
      end else if (main == main_read0) begin
        read_length <= tmp_0007;
      end else if (main == main_read) begin
        read_length <= tmp_0009;
      end
    end
  end



endmodule
