module axi_writer_32
#(
  parameter BUF_WIDTH = 32
)
(
  input  clk,
  input  reset,
  input [32-1 : 0] mem_din,
  output  mem_we,
  output [32-1 : 0] mem_address,
  output  mem_wclk,
  output [32-1 : 0] S_AXI_AWADDR,
  output [8-1 : 0] S_AXI_AWLEN,
  output  S_AXI_AWVALID,
  output [3-1 : 0] S_AXI_AWSIZE,
  output [2-1 : 0] S_AXI_AWBURST,
  output [4-1 : 0] S_AXI_AWCACHE,
  output [3-1 : 0] S_AXI_AWPROT,
  input  S_AXI_AWREADY,
  output [32-1 : 0] S_AXI_WDATA,
  output  S_AXI_WLAST,
  output  S_AXI_WVALID,
  input  S_AXI_WREADY,
  output [4-1 : 0] S_AXI_WSTRB,
  input [2-1 : 0] S_AXI_BRESP,
  input  S_AXI_BVALID,
  output  S_AXI_BREADY,
  input  request,
  output  busy,
  input [32-1 : 0] addr,
  input [32-1 : 0] len,
  output [8-1 : 0] debug
);

  wire  clk_sig;
  wire  reset_sig;
  wire [32-1 : 0] mem_din_sig;
  reg  mem_we_sig = 1'b0;
  reg [32-1 : 0] mem_address_sig = 0;
  wire  mem_wclk_sig;
  reg [32-1 : 0] S_AXI_AWADDR_sig = 0;
  reg [8-1 : 0] S_AXI_AWLEN_sig = 0;
  reg  S_AXI_AWVALID_sig = 1'b0;
  wire [3-1 : 0] S_AXI_AWSIZE_sig;
  wire [2-1 : 0] S_AXI_AWBURST_sig;
  wire [4-1 : 0] S_AXI_AWCACHE_sig;
  wire [3-1 : 0] S_AXI_AWPROT_sig;
  wire  S_AXI_AWREADY_sig;
  reg [32-1 : 0] S_AXI_WDATA_sig = 0;
  reg  S_AXI_WLAST_sig = 1'b0;
  reg  S_AXI_WVALID_sig = 1'b0;
  wire  S_AXI_WREADY_sig;
  wire [4-1 : 0] S_AXI_WSTRB_sig;
  wire [2-1 : 0] S_AXI_BRESP_sig;
  wire  S_AXI_BVALID_sig;
  reg  S_AXI_BREADY_sig = 1'b0;
  wire  request_sig;
  reg  busy_sig = 1'b0;
  wire [32-1 : 0] addr_sig;
  wire [32-1 : 0] len_sig;
  reg [8-1 : 0] debug_sig = 0;

  reg signed [32-1 : 0] write_counter = 0;
  reg signed [32-1 : 0] next_read_addr = 0;
  wire  tmp_0001;
  wire  tmp_0002;
  wire  tmp_0003;
  localparam main_IDLE = 32'd0;
  localparam main_init = 32'd1;
  localparam main_write0 = 32'd2;
  localparam main_write_pre = 32'd3;
  localparam main_write = 32'd4;
  localparam main_write_next = 32'd5;
  reg [31:0] main = main_IDLE;
  reg [31:0] main_prev = main_IDLE;
  reg signed [32-1 : 0] main_delay = 0;
  wire signed [32-1 : 0] tmp_0004;
  wire signed [8-1 : 0] tmp_0005;
  wire  tmp_0006;
  wire signed [32-1 : 0] tmp_0007;
  wire signed [32-1 : 0] tmp_0008;
  wire  tmp_0009;
  wire  tmp_0010;
  wire signed [32-1 : 0] tmp_0011;
  wire  tmp_0012;
  wire  tmp_0013;
  wire  tmp_0014;
  wire  tmp_0015;
  wire  tmp_0016;
  wire  tmp_0017;


  assign clk_sig = clk;
  assign reset_sig = reset;
  assign mem_din_sig = mem_din;
  assign mem_we = mem_we_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      mem_we_sig <= 1'b0;
    end else begin
      if (main == main_write0) begin
        mem_we_sig <= 1'b1;
      end else if (main == main_write_pre) begin
        mem_we_sig <= 1'b0;
      end else if (main == main_write) begin
        mem_we_sig <= tmp_0010;
      end else if (main == main_write_next) begin
        mem_we_sig <= 1'b0;
      end
    end
  end

  assign mem_address = mem_address_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      mem_address_sig <= 0;
    end else begin
      if (main == main_write0) begin
        mem_address_sig <= 0;
      end else if (main == main_write) begin
        mem_address_sig <= next_read_addr;
      end
    end
  end

  assign mem_wclk = mem_wclk_sig;
  assign mem_wclk_sig = clk_sig;

  assign S_AXI_AWADDR = S_AXI_AWADDR_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      S_AXI_AWADDR_sig <= 0;
    end else begin
      if (main == main_init) begin
        S_AXI_AWADDR_sig <= addr_sig;
      end
    end
  end

  assign S_AXI_AWLEN = S_AXI_AWLEN_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      S_AXI_AWLEN_sig <= 0;
    end else begin
      if (main == main_init) begin
        S_AXI_AWLEN_sig <= tmp_0005;
      end
    end
  end

  assign S_AXI_AWVALID = S_AXI_AWVALID_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      S_AXI_AWVALID_sig <= 1'b0;
    end else begin
      if (main == main_init) begin
        S_AXI_AWVALID_sig <= 1'b1;
      end else if (main == main_write0) begin
        S_AXI_AWVALID_sig <= tmp_0006;
      end
    end
  end

  assign S_AXI_AWSIZE = S_AXI_AWSIZE_sig;
  assign S_AXI_AWSIZE_sig = 3'b010;

  assign S_AXI_AWBURST = S_AXI_AWBURST_sig;
  assign S_AXI_AWBURST_sig = 2'b01;

  assign S_AXI_AWCACHE = S_AXI_AWCACHE_sig;
  assign S_AXI_AWCACHE_sig = 4'h3;

  assign S_AXI_AWPROT = S_AXI_AWPROT_sig;
  assign S_AXI_AWPROT_sig = 3'b000;

  assign S_AXI_AWREADY_sig = S_AXI_AWREADY;
  assign S_AXI_WDATA = S_AXI_WDATA_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      S_AXI_WDATA_sig <= 0;
    end else begin
      if (main == main_write) begin
        S_AXI_WDATA_sig <= mem_din_sig;
      end
    end
  end

  assign S_AXI_WLAST = S_AXI_WLAST_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      S_AXI_WLAST_sig <= 1'b0;
    end else begin
      if (main == main_write) begin
        S_AXI_WLAST_sig <= tmp_0009;
      end else if (main == main_write_next) begin
        S_AXI_WLAST_sig <= tmp_0012;
      end
    end
  end

  assign S_AXI_WVALID = S_AXI_WVALID_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      S_AXI_WVALID_sig <= 1'b0;
    end else begin
      if (main == main_write) begin
        S_AXI_WVALID_sig <= 1'b1;
      end else if (main == main_write_next) begin
        S_AXI_WVALID_sig <= tmp_0013;
      end
    end
  end

  assign S_AXI_WREADY_sig = S_AXI_WREADY;
  assign S_AXI_WSTRB = S_AXI_WSTRB_sig;
  assign S_AXI_WSTRB_sig = 4'hf;

  assign S_AXI_BRESP_sig = S_AXI_BRESP;
  assign S_AXI_BVALID_sig = S_AXI_BVALID;
  assign S_AXI_BREADY = S_AXI_BREADY_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      S_AXI_BREADY_sig <= 1'b0;
    end else begin
      if (main == main_IDLE) begin
        S_AXI_BREADY_sig <= 1'b1;
      end
    end
  end

  assign request_sig = request;
  assign busy = busy_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      busy_sig <= 1'b0;
    end else begin
      if (main == main_IDLE) begin
        busy_sig <= tmp_0014;
      end
    end
  end

  assign addr_sig = addr;
  assign len_sig = len;
  assign debug = debug_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      debug_sig <= 0;
    end else begin
      if (main == main_IDLE) begin
        debug_sig <= 8'h00;
      end else if (main == main_init) begin
        debug_sig <= 8'h01;
      end else if (main == main_write0) begin
        debug_sig <= 8'h02;
      end else if (main == main_write_pre) begin
        debug_sig <= 8'h03;
      end else if (main == main_write) begin
        debug_sig <= 8'h04;
      end else if (main == main_write_next) begin
        debug_sig <= 8'h05;
      end
    end
  end


  assign tmp_0001 = S_AXI_WREADY_sig == 1'b1 ? 1'b1 : 1'b0;
  assign tmp_0002 = write_counter == 0 ? 1'b1 : 1'b0;
  assign tmp_0003 = S_AXI_AWREADY_sig == 1'b1 ? 1'b1 : 1'b0;
  assign tmp_0004 = len_sig - 1;
  assign tmp_0005 = tmp_0004[32 - 32'sh00000018 - 1 : 0];
  assign tmp_0006 = tmp_0003 == 1'b1 ? 1'b0 : 1'b1;
  assign tmp_0007 = next_read_addr + 1;
  assign tmp_0008 = write_counter - 1;
  assign tmp_0009 = write_counter == 1 ? 1'b1 : 1'b0;
  assign tmp_0010 = tmp_0009 == 1'b1 ? 1'b0 : 1'b1;
  assign tmp_0011 = next_read_addr + 1;
  assign tmp_0012 = tmp_0001 == 1'b1 ? 1'b0 : S_AXI_WLAST_sig;
  assign tmp_0013 = tmp_0001 == 1'b1 ? 1'b0 : S_AXI_WVALID_sig;
  assign tmp_0014 = request_sig == 1'b1 ? 1'b1 : 1'b0;
  assign tmp_0015 = tmp_0001 & tmp_0002;
  assign tmp_0016 = ~tmp_0002;
  assign tmp_0017 = tmp_0001 & tmp_0016;

  always @(posedge clk) begin
    if(reset == 1'b1) begin
      main <= main_IDLE;
      main_delay <= 32'h0;
    end else begin
      case (main)
        main_IDLE : begin
          if (tmp_0014 == 1'b1) begin
            main <= main_init;
          end
        end
        main_init : begin
          main <= main_write0;
        end
        main_write0 : begin
          if (tmp_0003 == 1'b1) begin
            main <= main_write_pre;
          end
        end
        main_write_pre : begin
          main <= main_write;
        end
        main_write : begin
          main <= main_write_next;
        end
        main_write_next : begin
          if (tmp_0015 == 1'b1) begin
            main <= main_IDLE;
          end else if (tmp_0017 == 1'b1) begin
            main <= main_write;
          end
        end
      endcase
    end
  end


  always @(posedge clk) begin
    if(reset == 1'b1) begin
      write_counter <= 0;
    end else begin
      if (main == main_init) begin
        write_counter <= len_sig;
      end else if (main == main_write) begin
        write_counter <= tmp_0008;
      end
    end
  end

  always @(posedge clk) begin
    if(reset == 1'b1) begin
      next_read_addr <= 0;
    end else begin
      if (main == main_IDLE) begin
        next_read_addr <= 0;
      end else if (main == main_write0) begin
        next_read_addr <= tmp_0007;
      end else if (main == main_write) begin
        next_read_addr <= tmp_0011;
      end
    end
  end



endmodule
