module uplport
(
  input  UPLGlobalClk,
  input  UPLReset,
  output  ready,
  input  done,
  input signed [32-1 : 0] send_length,
  output signed [32-1 : 0] recv_length,
  output signed [32-1 : 0] data_length,
  input signed [32-1 : 0] data_address,
  output signed [32-1 : 0] data_dout,
  input signed [32-1 : 0] data_din,
  input  data_we,
  input  data_oe,
  input [32-1 : 0] UPLIn_Data,
  input  UPLIn_En,
  input  UPLIn_Request,
  output  UPLIn_Ack,
  output [32-1 : 0] UPLOut_Data,
  output  UPLOut_En,
  output  UPLOut_Request,
  input  UPLOut_Ack
);

  wire  UPLGlobalClk_sig;
  wire  UPLReset_sig;
  reg  ready_sig = 1'b0;
  wire  done_sig;
  wire signed [32-1 : 0] send_length_sig;
  reg signed [32-1 : 0] recv_length_sig = 0;
  wire signed [32-1 : 0] data_length_sig;
  wire signed [32-1 : 0] data_address_sig;
  wire signed [32-1 : 0] data_dout_sig;
  wire signed [32-1 : 0] data_din_sig;
  wire  data_we_sig;
  wire  data_oe_sig;
  wire [32-1 : 0] UPLIn_Data_sig;
  wire  UPLIn_En_sig;
  wire  UPLIn_Request_sig;
  reg  UPLIn_Ack_sig = 1'b0;
  reg [32-1 : 0] UPLOut_Data_sig = 0;
  reg  UPLOut_En_sig = 1'b0;
  reg  UPLOut_Request_sig = 1'b0;
  wire  UPLOut_Ack_sig;

  wire  U_clk;
  wire  U_reset;
  wire signed [32-1 : 0] U_length;
  wire signed [32-1 : 0] U_address;
  wire signed [32-1 : 0] U_din;
  wire signed [32-1 : 0] U_dout;
  wire  U_we;
  wire  U_oe;
  reg signed [32-1 : 0] U_address_b = 0;
  reg signed [32-1 : 0] U_din_b = 0;
  wire signed [32-1 : 0] U_dout_b;
  reg  U_we_b = 1'b0;
  reg  U_oe_b = 1'b0;
  reg signed [32-1 : 0] send_count = 0;
  reg signed [32-1 : 0] recv_count = 0;
  localparam main_IDLE = 32'd0;
  localparam main_RECV_WAIT = 32'd1;
  localparam main_RECV_DATA = 32'd2;
  localparam main_OPERATION = 32'd3;
  localparam main_SEND_WAIT = 32'd4;
  localparam main_MEM_WAIT = 32'd5;
  localparam main_SEND_DATA = 32'd6;
  reg [31:0] main = main_IDLE;
  reg [31:0] main_prev = main_IDLE;
  reg signed [32-1 : 0] main_delay = 0;
  wire  tmp_0001;
  wire signed [32-1 : 0] tmp_0002;
  wire signed [32-1 : 0] tmp_0003;
  wire signed [32-1 : 0] tmp_0004;
  wire signed [32-1 : 0] tmp_0005;
  wire signed [32-1 : 0] tmp_0006;
  wire signed [32-1 : 0] tmp_0007;
  wire signed [32-1 : 0] tmp_0008;
  wire signed [32-1 : 0] tmp_0009;
  wire signed [32-1 : 0] tmp_0010;
  wire  tmp_0011;
  wire  tmp_0012;
  wire  tmp_0013;
  wire  tmp_0014;
  reg  done_reg = 1'b0;
  wire  tmp_0015;
  wire  tmp_0016;
  wire  tmp_0017;
  wire  tmp_0018;
  wire  tmp_0019;
  wire  tmp_0020;


  assign UPLGlobalClk_sig = UPLGlobalClk;
  assign UPLReset_sig = UPLReset;
  assign ready = ready_sig;
  always @(posedge UPLGlobalClk) begin
    if(UPLReset == 1'b1) begin
      ready_sig <= 1'b0;
    end else begin
      if (main == main_OPERATION) begin
        ready_sig <= 1'b1;
      end else begin
        ready_sig <= 1'b0;
      end
    end
  end

  assign done_sig = done;
  assign send_length_sig = send_length;
  assign recv_length = recv_length_sig;
  assign data_length = data_length_sig;
  assign data_length_sig = U_length;

  assign data_address_sig = data_address;
  assign data_dout = data_dout_sig;
  assign data_dout_sig = U_dout;

  assign data_din_sig = data_din;
  assign data_we_sig = data_we;
  assign data_oe_sig = data_oe;
  assign UPLIn_Data_sig = UPLIn_Data;
  assign UPLIn_En_sig = UPLIn_En;
  assign UPLIn_Request_sig = UPLIn_Request;
  assign UPLIn_Ack = UPLIn_Ack_sig;
  always @(posedge UPLGlobalClk) begin
    if(UPLReset == 1'b1) begin
      UPLIn_Ack_sig <= 1'b0;
    end else begin
      if (main == main_RECV_WAIT) begin
        UPLIn_Ack_sig <= tmp_0001;
      end else begin
        UPLIn_Ack_sig <= 1'b0;
      end
    end
  end

  assign UPLOut_Data = UPLOut_Data_sig;
  always @(posedge UPLGlobalClk) begin
    if(UPLReset == 1'b1) begin
      UPLOut_Data_sig <= 0;
    end else begin
      if (main == main_SEND_DATA) begin
        UPLOut_Data_sig <= U_dout;
      end
    end
  end

  assign UPLOut_En = UPLOut_En_sig;
  always @(posedge UPLGlobalClk) begin
    if(UPLReset == 1'b1) begin
      UPLOut_En_sig <= 1'b0;
    end else begin
      if (main == main_SEND_DATA) begin
        UPLOut_En_sig <= 1'b1;
      end else begin
        UPLOut_En_sig <= 1'b0;
      end
    end
  end

  assign UPLOut_Request = UPLOut_Request_sig;
  always @(posedge UPLGlobalClk) begin
    if(UPLReset == 1'b1) begin
      UPLOut_Request_sig <= 1'b0;
    end else begin
      if (main == main_SEND_WAIT) begin
        UPLOut_Request_sig <= 1'b1;
      end else begin
        UPLOut_Request_sig <= 1'b0;
      end
    end
  end

  assign UPLOut_Ack_sig = UPLOut_Ack;

  assign tmp_0001 = ~UPLIn_En_sig;
  assign tmp_0002 = UPLIn_En_sig == 1'b1 ? 32'sh00000001 : 32'sh00000000;
  assign tmp_0003 = U_address_b + 32'sh00000001;
  assign tmp_0004 = UPLIn_En_sig == 1'b1 ? tmp_0003 : U_address_b;
  assign tmp_0005 = recv_count + 32'sh00000001;
  assign tmp_0006 = UPLIn_En_sig == 1'b1 ? tmp_0005 : recv_count;
  assign tmp_0007 = send_length_sig;
  assign tmp_0008 = U_address_b + 32'sh00000001;
  assign tmp_0009 = send_count - 32'sh00000001;
  assign tmp_0010 = U_address_b + 32'sh00000001;
  assign tmp_0011 = UPLIn_En_sig == 1'b1 ? 1'b1 : 1'b0;
  assign tmp_0012 = UPLIn_En_sig == 1'b0 ? 1'b1 : 1'b0;
  assign tmp_0013 = send_length_sig > 32'sh00000000 ? 1'b1 : 1'b0;
  assign tmp_0014 = ~tmp_0013;
  assign tmp_0015 = ~done_reg;
  assign tmp_0016 = done_sig & tmp_0015;
  assign tmp_0017 = tmp_0016 & tmp_0013;
  assign tmp_0018 = tmp_0016 & tmp_0014;
  assign tmp_0019 = UPLOut_Ack_sig == 1'b1 ? 1'b1 : 1'b0;
  assign tmp_0020 = send_count == 32'sh00000001 ? 1'b1 : 1'b0;

  always @(posedge UPLGlobalClk) begin
    if(UPLReset == 1'b1) begin
      main <= main_IDLE;
      main_delay <= 32'h0;
    end else begin
      case (main)
        main_IDLE : begin
          main <= main_RECV_WAIT;
        end
        main_RECV_WAIT : begin
          if (tmp_0011 == 1'b1) begin
            main <= main_RECV_DATA;
          end
        end
        main_RECV_DATA : begin
          if (tmp_0012 == 1'b1) begin
            main <= main_OPERATION;
          end
        end
        main_OPERATION : begin
          if (tmp_0017 == 1'b1) begin
            main <= main_SEND_WAIT;
          end else if (tmp_0018 == 1'b1) begin
            main <= main_IDLE;
          end
        end
        main_SEND_WAIT : begin
          if (tmp_0019 == 1'b1) begin
            main <= main_MEM_WAIT;
          end
        end
        main_MEM_WAIT : begin
          main <= main_SEND_DATA;
        end
        main_SEND_DATA : begin
          if (tmp_0020 == 1'b1) begin
            main <= main_IDLE;
          end
        end
      endcase
      done_reg <= done_sig;
    end
  end


  assign U_clk = UPLGlobalClk_sig;

  assign U_reset = UPLReset_sig;

  assign U_address = data_address_sig;

  assign U_din = data_din_sig;

  assign U_we = data_we_sig;

  assign U_oe = data_oe_sig;

  always @(posedge UPLGlobalClk) begin
    if(UPLReset == 1'b1) begin
      U_address_b <= 0;
    end else begin
      if (main == main_RECV_WAIT) begin
        U_address_b <= 32'sh00000000;
      end else if (main == main_RECV_DATA) begin
        U_address_b <= tmp_0004;
      end else if (main == main_SEND_WAIT) begin
        U_address_b <= 32'sh00000000;
      end else if (main == main_MEM_WAIT) begin
        U_address_b <= tmp_0008;
      end else if (main == main_SEND_DATA) begin
        U_address_b <= tmp_0010;
      end
    end
  end

  always @(posedge UPLGlobalClk) begin
    if(UPLReset == 1'b1) begin
      send_count <= 0;
    end else begin
      if (main == main_SEND_WAIT) begin
        send_count <= tmp_0007;
      end else if (main == main_SEND_DATA) begin
        send_count <= tmp_0009;
      end
    end
  end

  always @(posedge UPLGlobalClk) begin
    if(UPLReset == 1'b1) begin
      recv_count <= 0;
    end else begin
      if (main == main_RECV_WAIT) begin
        recv_count <= tmp_0002;
      end else if (main == main_RECV_DATA) begin
        recv_count <= tmp_0006;
      end
    end
  end


  dualportram
  #(
    .WIDTH(32),
    .DEPTH(10),
    .WORDS(1024)
  )
  U
  (
    .clk(UPLGlobalClk),
    .reset(UPLReset),
    .length(U_length),
    .address(U_address),
    .din(U_din),
    .dout(U_dout),
    .we(U_we),
    .oe(U_oe),
    .address_b(U_address_b),
    .din_b(U_din_b),
    .dout_b(U_dout_b),
    .we_b(U_we_b),
    .oe_b(U_oe_b)
  );


endmodule
