library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity axi_memiface_128_bram_ext_ctrl is
  port (
    clk : in std_logic;
    reset : in std_logic;
    data_address_external : in std_logic_vector(32-1 downto 0);
    data_din_external : in std_logic_vector(128-1 downto 0);
    data_dout_external : out std_logic_vector(128-1 downto 0);
    data_we_external : in std_logic;
    data_oe_external : in std_logic;
    ext_ctrl : in std_logic;
    ext_ctrl_flag : out std_logic;
    data_length : out std_logic_vector(32-1 downto 0);
    busy : out std_logic;
    write_kick : in std_logic;
    read_kick : in std_logic;
    burst_size : in std_logic_vector(32-1 downto 0);
    axi_addr : in std_logic_vector(32-1 downto 0);
    forbid : in std_logic;
    axi_reader_ARADDR : out std_logic_vector(32-1 downto 0);
    axi_reader_ARLEN : out std_logic_vector(8-1 downto 0);
    axi_reader_ARVALID : out std_logic;
    axi_reader_ARREADY : in std_logic;
    axi_reader_ARSIZE : out std_logic_vector(3-1 downto 0);
    axi_reader_ARBURST : out std_logic_vector(2-1 downto 0);
    axi_reader_ARCACHE : out std_logic_vector(4-1 downto 0);
    axi_reader_ARPROT : out std_logic_vector(3-1 downto 0);
    axi_reader_RDATA : in std_logic_vector(128-1 downto 0);
    axi_reader_RRESP : in std_logic_vector(2-1 downto 0);
    axi_reader_RLAST : in std_logic;
    axi_reader_RVALID : in std_logic;
    axi_reader_RREADY : out std_logic;
    axi_writer_AWADDR : out std_logic_vector(32-1 downto 0);
    axi_writer_AWLEN : out std_logic_vector(8-1 downto 0);
    axi_writer_AWVALID : out std_logic;
    axi_writer_AWSIZE : out std_logic_vector(3-1 downto 0);
    axi_writer_AWBURST : out std_logic_vector(2-1 downto 0);
    axi_writer_AWCACHE : out std_logic_vector(4-1 downto 0);
    axi_writer_AWPROT : out std_logic_vector(3-1 downto 0);
    axi_writer_AWREADY : in std_logic;
    axi_writer_WDATA : out std_logic_vector(128-1 downto 0);
    axi_writer_WLAST : out std_logic;
    axi_writer_WVALID : out std_logic;
    axi_writer_WREADY : in std_logic;
    axi_writer_WSTRB : out std_logic_vector(16-1 downto 0);
    axi_writer_BRESP : in std_logic_vector(2-1 downto 0);
    axi_writer_BVALID : in std_logic;
    axi_writer_BREADY : out std_logic
  );
end axi_memiface_128_bram_ext_ctrl;

architecture RTL of axi_memiface_128_bram_ext_ctrl is

  attribute mark_debug : string;
  attribute keep : string;
  attribute S : string;

  component dualportram
    generic (
  WIDTH : integer := 128;
  DEPTH : integer := 10;
  WORDS : integer := 1024
    );
    port (
      clk : in std_logic;
      reset : in std_logic;
      length : out signed(31 downto 0);
      address : in signed(31 downto 0);
      din : in signed(WIDTH-1 downto 0);
      dout : out signed(WIDTH-1 downto 0);
      we : in std_logic;
      oe : in std_logic;
      address_b : in signed(31 downto 0);
      din_b : in signed(WIDTH-1 downto 0);
      dout_b : out signed(WIDTH-1 downto 0);
      we_b : in std_logic;
      oe_b : in std_logic
    );
  end component dualportram;
  component simple_fifo_128
    port (
      clk : in std_logic;
      reset : in std_logic;
      we : in std_logic;
      din : in std_logic_vector(128-1 downto 0);
      re : in std_logic;
      dout : out std_logic_vector(128-1 downto 0);
      empty : out std_logic;
      full : out std_logic;
      count : out std_logic_vector(32-1 downto 0)
    );
  end component simple_fifo_128;

  signal clk_sig : std_logic := '0';
  signal reset_sig : std_logic := '0';
  signal data_address_external_sig : std_logic_vector(32-1 downto 0) := (others => '0');
  signal data_din_external_sig : std_logic_vector(128-1 downto 0) := (others => '0');
  signal data_dout_external_sig : std_logic_vector(128-1 downto 0) := (others => '0');
  signal data_we_external_sig : std_logic := '0';
  signal data_oe_external_sig : std_logic := '0';
  signal ext_ctrl_sig : std_logic := '0';
  signal ext_ctrl_flag_sig : std_logic := '0';
  signal data_length_sig : std_logic_vector(32-1 downto 0) := (others => '0');
  signal busy_sig : std_logic := '0';
  signal write_kick_sig : std_logic := '0';
  signal read_kick_sig : std_logic := '0';
  signal burst_size_sig : std_logic_vector(32-1 downto 0) := (others => '0');
  signal axi_addr_sig : std_logic_vector(32-1 downto 0) := (others => '0');
  signal forbid_sig : std_logic := '0';
  signal axi_reader_ARADDR_sig : std_logic_vector(32-1 downto 0) := (others => '0');
  signal axi_reader_ARLEN_sig : std_logic_vector(8-1 downto 0) := (others => '0');
  signal axi_reader_ARVALID_sig : std_logic := '0';
  signal axi_reader_ARREADY_sig : std_logic := '0';
  signal axi_reader_ARSIZE_sig : std_logic_vector(3-1 downto 0) := (others => '0');
  signal axi_reader_ARBURST_sig : std_logic_vector(2-1 downto 0) := (others => '0');
  signal axi_reader_ARCACHE_sig : std_logic_vector(4-1 downto 0) := (others => '0');
  signal axi_reader_ARPROT_sig : std_logic_vector(3-1 downto 0) := (others => '0');
  signal axi_reader_RDATA_sig : std_logic_vector(128-1 downto 0) := (others => '0');
  signal axi_reader_RRESP_sig : std_logic_vector(2-1 downto 0) := (others => '0');
  signal axi_reader_RLAST_sig : std_logic := '0';
  signal axi_reader_RVALID_sig : std_logic := '0';
  signal axi_reader_RREADY_sig : std_logic := '0';
  signal axi_writer_AWADDR_sig : std_logic_vector(32-1 downto 0) := (others => '0');
  signal axi_writer_AWLEN_sig : std_logic_vector(8-1 downto 0) := (others => '0');
  signal axi_writer_AWVALID_sig : std_logic := '0';
  signal axi_writer_AWSIZE_sig : std_logic_vector(3-1 downto 0) := (others => '0');
  signal axi_writer_AWBURST_sig : std_logic_vector(2-1 downto 0) := (others => '0');
  signal axi_writer_AWCACHE_sig : std_logic_vector(4-1 downto 0) := (others => '0');
  signal axi_writer_AWPROT_sig : std_logic_vector(3-1 downto 0) := (others => '0');
  signal axi_writer_AWREADY_sig : std_logic := '0';
  signal axi_writer_WDATA_sig : std_logic_vector(128-1 downto 0) := (others => '0');
  signal axi_writer_WLAST_sig : std_logic := '0';
  signal axi_writer_WVALID_sig : std_logic := '0';
  signal axi_writer_WREADY_sig : std_logic := '0';
  signal axi_writer_WSTRB_sig : std_logic_vector(16-1 downto 0) := (others => '0');
  signal axi_writer_BRESP_sig : std_logic_vector(2-1 downto 0) := (others => '0');
  signal axi_writer_BVALID_sig : std_logic := '0';
  signal axi_writer_BREADY_sig : std_logic := '0';

  signal data_address_act : signed(32-1 downto 0) := (others => '0');
  signal data_din_act : signed(128-1 downto 0) := (others => '0');
  signal data_dout_act : signed(128-1 downto 0) := (others => '0');
  signal data_we_act : std_logic := '0';
  signal data_oe_act : std_logic := '0';
  signal BUF_clk : std_logic := '0';
  signal BUF_reset : std_logic := '0';
  signal BUF_length : signed(32-1 downto 0) := (others => '0');
  signal BUF_address : signed(32-1 downto 0) := (others => '0');
  signal BUF_din : signed(128-1 downto 0) := (others => '0');
  signal BUF_dout : signed(128-1 downto 0) := (others => '0');
  signal BUF_we : std_logic := '0';
  signal BUF_oe : std_logic := '0';
  signal BUF_address_b : signed(32-1 downto 0) := (others => '0');
  signal BUF_din_b : signed(128-1 downto 0) := (others => '0');
  signal BUF_dout_b : signed(128-1 downto 0) := (others => '0');
  signal BUF_we_b : std_logic := '0';
  signal BUF_oe_b : std_logic := '0';
  signal read_state_busy : std_logic := '0';
  signal write_state_busy : std_logic := '0';
  signal tmp_0001 : std_logic := '0';
  signal tmp_0002 : std_logic := '0';
  signal hdl_write_kick_d : std_logic := '0';
  signal tmp_0003 : std_logic := '0';
  signal tmp_0004 : std_logic := '0';
  signal hdl_read_kick_d : std_logic := '0';
  signal tmp_0005 : std_logic := '0';
  signal tmp_0006 : std_logic := '0';
  signal tmp_0007 : std_logic := '0';
  signal tmp_0008 : std_logic := '0';
  type Type_axi is (
    axi_IDLE,
    axi_sjr_tmp_state_0000,
    axi_sjr_tmp_state_0001,
    axi_sjr_tmp_state_0002,
    axi_sjr_tmp_state_0003,
    axi_sjr_tmp_state_0004,
    axi_sjr_tmp_state_0005,
    axi_sjr_tmp_state_0006  
  );
  signal axi : Type_axi := axi_IDLE;
  signal axi_prev : Type_axi := axi_IDLE;
  signal axi_delay : signed(32-1 downto 0) := (others => '0');
  signal sjr_generate_U_0009_clk : std_logic := '0';
  signal sjr_generate_U_0009_reset : std_logic := '0';
  signal sjr_generate_U_0009_we : std_logic := '0';
  signal sjr_generate_U_0009_din : std_logic_vector(128-1 downto 0) := (others => '0');
  signal sjr_generate_U_0009_re : std_logic := '0';
  signal sjr_generate_U_0009_dout : std_logic_vector(128-1 downto 0) := (others => '0');
  signal sjr_generate_U_0009_empty : std_logic := '0';
  signal sjr_generate_U_0009_full : std_logic := '0';
  signal sjr_generate_U_0009_count : std_logic_vector(32-1 downto 0) := (others => '0');
  signal tmp_0010 : std_logic := '0';
  signal tmp_0011 : std_logic := '0';
  signal tmp_0012 : std_logic_vector(128-1 downto 0) := (others => '0');
  signal tmp_0013 : std_logic_vector(128-1 downto 0) := (others => '0');
  signal tmp_0014 : std_logic := '0';
  signal tmp_0015 : std_logic := '0';
  signal tmp_0016 : std_logic_vector(32-1 downto 0) := (others => '0');
  signal tmp_0017 : std_logic_vector(32-1 downto 0) := (others => '0');
  signal tmp_0018 : std_logic := '0';
  signal tmp_0019 : std_logic_vector(32-1 downto 0) := (others => '0');
  signal tmp_0020 : std_logic_vector(32-1 downto 0) := (others => '0');
  signal tmp_0021 : signed(32-1 downto 0) := (others => '0');
  signal tmp_0022 : signed(32-1 downto 0) := (others => '0');
  signal tmp_0023 : signed(32-1 downto 0) := (others => '0');
  signal tmp_0024 : std_logic := '0';
  signal tmp_0025 : std_logic := '0';
  signal tmp_0026 : std_logic := '0';
  signal tmp_0027 : std_logic_vector(32-1 downto 0) := (others => '0');
  signal tmp_0028 : std_logic_vector(8-1 downto 0) := (others => '0');
  signal tmp_0029 : std_logic := '0';
  signal tmp_0030 : signed(32-1 downto 0) := (others => '0');
  signal tmp_0031 : std_logic := '0';
  signal tmp_0032 : std_logic := '0';
  signal tmp_0033 : std_logic := '0';
  signal tmp_0034 : std_logic := '0';
  signal tmp_0035 : std_logic := '0';
  signal tmp_0036 : std_logic := '0';
  signal tmp_0037 : std_logic := '0';
  signal tmp_0038 : std_logic := '0';
  signal tmp_0039 : std_logic := '0';
  signal tmp_0040 : std_logic := '0';
  signal tmp_0041 : std_logic := '0';
  signal tmp_0042 : std_logic := '0';
  signal tmp_0043 : std_logic := '0';
  signal tmp_0044 : std_logic := '0';
  signal tmp_0045 : std_logic := '0';
  signal tmp_0046 : std_logic := '0';
  signal tmp_0047 : std_logic := '0';
  signal tmp_0048 : std_logic := '0';
  signal tmp_0049 : std_logic := '0';
  signal tmp_0050 : std_logic := '0';
  signal tmp_0051 : std_logic := '0';
  signal tmp_0052 : std_logic := '0';
  signal tmp_0053 : std_logic := '0';
  signal tmp_0054 : signed(32-1 downto 0) := (others => '0');
  signal tmp_0055 : std_logic := '0';
  signal tmp_0056 : std_logic := '0';
  signal tmp_0057 : std_logic := '0';
  signal tmp_0058 : signed(32-1 downto 0) := (others => '0');
  signal tmp_0059 : signed(32-1 downto 0) := (others => '0');
  signal tmp_0060 : std_logic_vector(32-1 downto 0) := (others => '0');
  signal tmp_0061 : std_logic_vector(8-1 downto 0) := (others => '0');
  signal tmp_0062 : std_logic := '0';
  signal tmp_0063 : std_logic := '0';
  signal tmp_0064 : signed(32-1 downto 0) := (others => '0');
  signal tmp_0065 : signed(32-1 downto 0) := (others => '0');
  signal tmp_0066 : signed(32-1 downto 0) := (others => '0');
  signal tmp_0067 : signed(32-1 downto 0) := (others => '0');
  signal tmp_0068 : std_logic := '0';

begin

  clk_sig <= clk;
  reset_sig <= reset;
  data_address_external_sig <= data_address_external;
  data_din_external_sig <= data_din_external;
  data_dout_external <= data_dout_external_sig;
  data_dout_external_sig <= std_logic_vector(data_dout_act);

  data_we_external_sig <= data_we_external;
  data_oe_external_sig <= data_oe_external;
  ext_ctrl_sig <= ext_ctrl;
  ext_ctrl_flag <= ext_ctrl_flag_sig;
  ext_ctrl_flag_sig <= ext_ctrl_sig;

  data_length <= data_length_sig;
  data_length_sig <= std_logic_vector(BUF_length);

  busy <= busy_sig;
  busy_sig <= tmp_0002;

  write_kick_sig <= write_kick;
  read_kick_sig <= read_kick;
  burst_size_sig <= burst_size;
  axi_addr_sig <= axi_addr;
  forbid_sig <= forbid;
  axi_reader_ARADDR <= axi_reader_ARADDR_sig;
  process(clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        axi_reader_ARADDR_sig <= (others => '0');
      else
        if axi = axi_IDLE then
          axi_reader_ARADDR_sig <= axi_addr_sig;
        end if;
      end if;
    end if;
  end process;

  axi_reader_ARLEN <= axi_reader_ARLEN_sig;
  process(clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        axi_reader_ARLEN_sig <= (others => '0');
      else
        if axi = axi_IDLE then
          axi_reader_ARLEN_sig <= tmp_0061;
        end if;
      end if;
    end if;
  end process;

  axi_reader_ARVALID <= axi_reader_ARVALID_sig;
  process(clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        axi_reader_ARVALID_sig <= '0';
      else
        if axi = axi_IDLE then
          axi_reader_ARVALID_sig <= tmp_0008;
        elsif axi = axi_sjr_tmp_state_0005 then
          axi_reader_ARVALID_sig <= tmp_0062;
        end if;
      end if;
    end if;
  end process;

  axi_reader_ARREADY_sig <= axi_reader_ARREADY;
  axi_reader_ARSIZE <= axi_reader_ARSIZE_sig;
  axi_reader_ARSIZE_sig <= "100";

  axi_reader_ARBURST <= axi_reader_ARBURST_sig;
  axi_reader_ARBURST_sig <= "01";

  axi_reader_ARCACHE <= axi_reader_ARCACHE_sig;
  axi_reader_ARCACHE_sig <= X"3";

  axi_reader_ARPROT <= axi_reader_ARPROT_sig;
  axi_reader_ARPROT_sig <= "000";

  axi_reader_RDATA_sig <= axi_reader_RDATA;
  axi_reader_RRESP_sig <= axi_reader_RRESP;
  axi_reader_RLAST_sig <= axi_reader_RLAST;
  axi_reader_RVALID_sig <= axi_reader_RVALID;
  axi_reader_RREADY <= axi_reader_RREADY_sig;
  process(clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        axi_reader_RREADY_sig <= '0';
      else
        if axi = axi_IDLE then
          axi_reader_RREADY_sig <= '1';
        end if;
      end if;
    end if;
  end process;

  axi_writer_AWADDR <= axi_writer_AWADDR_sig;
  process(clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        axi_writer_AWADDR_sig <= (others => '0');
      else
        if axi = axi_IDLE then
          axi_writer_AWADDR_sig <= axi_addr_sig;
        end if;
      end if;
    end if;
  end process;

  axi_writer_AWLEN <= axi_writer_AWLEN_sig;
  process(clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        axi_writer_AWLEN_sig <= (others => '0');
      else
        if axi = axi_IDLE then
          axi_writer_AWLEN_sig <= tmp_0028;
        end if;
      end if;
    end if;
  end process;

  axi_writer_AWVALID <= axi_writer_AWVALID_sig;
  process(clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        axi_writer_AWVALID_sig <= '0';
      else
        if axi = axi_IDLE then
          axi_writer_AWVALID_sig <= tmp_0004;
        elsif axi = axi_sjr_tmp_state_0000 then
          axi_writer_AWVALID_sig <= tmp_0029;
        end if;
      end if;
    end if;
  end process;

  axi_writer_AWSIZE <= axi_writer_AWSIZE_sig;
  axi_writer_AWSIZE_sig <= "100";

  axi_writer_AWBURST <= axi_writer_AWBURST_sig;
  axi_writer_AWBURST_sig <= "01";

  axi_writer_AWCACHE <= axi_writer_AWCACHE_sig;
  axi_writer_AWCACHE_sig <= X"3";

  axi_writer_AWPROT <= axi_writer_AWPROT_sig;
  axi_writer_AWPROT_sig <= "000";

  axi_writer_AWREADY_sig <= axi_writer_AWREADY;
  axi_writer_WDATA <= axi_writer_WDATA_sig;
  axi_writer_WDATA_sig <= tmp_0013;

  axi_writer_WLAST <= axi_writer_WLAST_sig;
  process(clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        axi_writer_WLAST_sig <= '0';
      else
        if axi = axi_IDLE then
          axi_writer_WLAST_sig <= '0';
        elsif axi = axi_sjr_tmp_state_0003 then
          axi_writer_WLAST_sig <= tmp_0044;
        elsif axi = axi_sjr_tmp_state_0004 then
          axi_writer_WLAST_sig <= '0';
        end if;
      end if;
    end if;
  end process;

  axi_writer_WVALID <= axi_writer_WVALID_sig;
  process(clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        axi_writer_WVALID_sig <= '0';
      else
        if axi = axi_IDLE then
          axi_writer_WVALID_sig <= '0';
        elsif axi = axi_sjr_tmp_state_0003 then
          axi_writer_WVALID_sig <= tmp_0050;
        elsif axi = axi_sjr_tmp_state_0004 then
          axi_writer_WVALID_sig <= '0';
        end if;
      end if;
    end if;
  end process;

  axi_writer_WREADY_sig <= axi_writer_WREADY;
  axi_writer_WSTRB <= axi_writer_WSTRB_sig;
  axi_writer_WSTRB_sig <= X"ffff";

  axi_writer_BRESP_sig <= axi_writer_BRESP;
  axi_writer_BVALID_sig <= axi_writer_BVALID;
  axi_writer_BREADY <= axi_writer_BREADY_sig;
  process(clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        axi_writer_BREADY_sig <= '0';
      else
        if axi = axi_IDLE then
          axi_writer_BREADY_sig <= '1';
        end if;
      end if;
    end if;
  end process;


  -- expressions
  tmp_0001 <= write_state_busy or read_state_busy;
  tmp_0002 <= tmp_0001 or forbid_sig;
  tmp_0003 <= not hdl_write_kick_d;
  tmp_0004 <= write_kick_sig and tmp_0003;
  tmp_0005 <= not tmp_0004;
  tmp_0006 <= not hdl_read_kick_d;
  tmp_0007 <= read_kick_sig and tmp_0006;
  tmp_0008 <= tmp_0005 and tmp_0007;
  tmp_0018 <= '1' when signed(tmp_0017) = 0 else '0';
  tmp_0019 <= std_logic_vector(signed(tmp_0017) - 1);
  tmp_0020 <= (others => '0') when tmp_0018 = '1' else tmp_0019;
  tmp_0022 <= BUF_address_b + 1;
  tmp_0023 <= tmp_0021 - 1;
  tmp_0025 <= tmp_0024 and axi_writer_WVALID_sig;
  tmp_0026 <= tmp_0025 and axi_writer_WREADY_sig;
  tmp_0027 <= std_logic_vector(signed(burst_size_sig) - 1);
  tmp_0028 <= tmp_0027(32 - 24 - 1 downto 0);
  tmp_0029 <= not axi_writer_AWREADY_sig;
  tmp_0030 <= tmp_0022 when axi_writer_AWREADY_sig = '1' else BUF_address_b;
  tmp_0031 <= '1' when signed(tmp_0017) > 0 else '0';
  tmp_0032 <= '1' when tmp_0031 = '1' else '0';
  tmp_0033 <= '1' when signed(tmp_0017) > 0 else '0';
  tmp_0034 <= '1' when tmp_0033 = '1' else '0';
  tmp_0035 <= '1' when signed(tmp_0017) > 0 else '0';
  tmp_0036 <= '1' when tmp_0035 = '1' else '0';
  tmp_0037 <= '1' when tmp_0021 = 1 else '0';
  tmp_0038 <= axi_writer_WVALID_sig and axi_writer_WREADY_sig;
  tmp_0039 <= '0' when tmp_0038 = '1' else '1';
  tmp_0040 <= '1' when tmp_0021 = 2 else '0';
  tmp_0041 <= axi_writer_WVALID_sig and axi_writer_WREADY_sig;
  tmp_0042 <= '1' when tmp_0041 = '1' else '0';
  tmp_0043 <= tmp_0042 when tmp_0040 = '1' else '0';
  tmp_0044 <= tmp_0039 when tmp_0037 = '1' else tmp_0043;
  tmp_0045 <= '1' when tmp_0021 > 1 else '0';
  tmp_0046 <= '1' when tmp_0021 = 1 else '0';
  tmp_0047 <= axi_writer_WVALID_sig and axi_writer_WREADY_sig;
  tmp_0048 <= tmp_0046 and tmp_0047;
  tmp_0049 <= '0' when tmp_0048 = '1' else '1';
  tmp_0050 <= '1' when tmp_0045 = '1' else tmp_0049;
  tmp_0051 <= axi_writer_WVALID_sig and axi_writer_WREADY_sig;
  tmp_0052 <= '1' when tmp_0021 > 0 else '0';
  tmp_0053 <= tmp_0051 and tmp_0052;
  tmp_0054 <= tmp_0023 when tmp_0053 = '1' else tmp_0021;
  tmp_0055 <= '1' when tmp_0021 = 1 else '0';
  tmp_0056 <= axi_writer_WVALID_sig and axi_writer_WREADY_sig;
  tmp_0057 <= tmp_0055 and tmp_0056;
  tmp_0060 <= std_logic_vector(signed(burst_size_sig) - 1);
  tmp_0061 <= tmp_0060(32 - 24 - 1 downto 0);
  tmp_0062 <= not axi_reader_ARREADY_sig;
  tmp_0063 <= '1' when tmp_0058 = 1 else '0';
  tmp_0064 <= tmp_0058 - 1;
  tmp_0065 <= tmp_0059 + 1;
  tmp_0066 <= tmp_0064 when axi_reader_RVALID_sig = '1' else tmp_0058;
  tmp_0067 <= tmp_0065 when axi_reader_RVALID_sig = '1' else tmp_0059;
  tmp_0068 <= axi_reader_RVALID_sig and tmp_0063;

  -- sequencers
  process (clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        axi <= axi_IDLE;
        axi_delay <= (others => '0');
      else
        case (axi) is
          when axi_IDLE => 
            if tmp_0004 = '1' then
              axi <= axi_sjr_tmp_state_0000;
            elsif tmp_0008 = '1' then
              axi <= axi_sjr_tmp_state_0005;
            end if;
          when axi_sjr_tmp_state_0000 => 
            if axi_writer_AWREADY_sig = '1' then
              axi <= axi_sjr_tmp_state_0001;
            end if;
          when axi_sjr_tmp_state_0001 => 
            axi <= axi_sjr_tmp_state_0002;
          when axi_sjr_tmp_state_0002 => 
            axi <= axi_sjr_tmp_state_0003;
          when axi_sjr_tmp_state_0003 => 
            if tmp_0057 = '1' then
              axi <= axi_sjr_tmp_state_0004;
            end if;
          when axi_sjr_tmp_state_0004 => 
            if axi_writer_BVALID_sig = '1' then
              axi <= axi_IDLE;
            end if;
          when axi_sjr_tmp_state_0005 => 
            if axi_reader_ARREADY_sig = '1' then
              axi <= axi_sjr_tmp_state_0006;
            end if;
          when axi_sjr_tmp_state_0006 => 
            if tmp_0068 = '1' then
              axi <= axi_IDLE;
            end if;
          when others => null;
        end case;
        hdl_write_kick_d <= write_kick_sig;
        hdl_read_kick_d <= read_kick_sig;
      end if;
    end if;
  end process;


  data_address_act <= signed(data_address_external_sig);

  data_din_act <= signed(data_din_external_sig);

  data_dout_act <= BUF_dout;

  data_we_act <= data_we_external_sig;

  data_oe_act <= data_oe_external_sig;

  BUF_clk <= clk_sig;

  BUF_reset <= reset_sig;

  BUF_address <= data_address_act;

  BUF_din <= data_din_act;

  BUF_we <= data_we_act;

  BUF_oe <= data_oe_act;

  process(clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        BUF_address_b <= (others => '0');
      else
        if axi = axi_sjr_tmp_state_0000 then
          BUF_address_b <= tmp_0030;
        elsif axi = axi_sjr_tmp_state_0001 then
          BUF_address_b <= tmp_0022;
        elsif axi = axi_sjr_tmp_state_0002 then
          BUF_address_b <= tmp_0022;
        elsif axi = axi_sjr_tmp_state_0003 then
          BUF_address_b <= tmp_0022;
        elsif axi = axi_IDLE then
          BUF_address_b <= (others => '0');
        elsif axi = axi_sjr_tmp_state_0006 then
          BUF_address_b <= tmp_0059;
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        BUF_din_b <= (others => '0');
      else
        if axi = axi_sjr_tmp_state_0006 then
          BUF_din_b <= signed(axi_reader_RDATA_sig);
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        BUF_we_b <= '0';
      else
        if axi = axi_sjr_tmp_state_0006 then
          BUF_we_b <= axi_reader_RVALID_sig;
        else
          BUF_we_b <= '0';
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        BUF_oe_b <= '0';
      else
        if axi = axi_IDLE then
          BUF_oe_b <= tmp_0004;
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        read_state_busy <= '0';
      else
        if axi = axi_IDLE then
          read_state_busy <= tmp_0008;
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        write_state_busy <= '0';
      else
        if axi = axi_IDLE then
          write_state_busy <= tmp_0004;
        end if;
      end if;
    end if;
  end process;

  sjr_generate_U_0009_clk <= clk_sig;

  sjr_generate_U_0009_reset <= reset_sig;

  sjr_generate_U_0009_we <= tmp_0010;

  sjr_generate_U_0009_din <= tmp_0012;

  sjr_generate_U_0009_re <= tmp_0011;

  process(clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        tmp_0010 <= '0';
      else
        if axi = axi_sjr_tmp_state_0001 then
          tmp_0010 <= tmp_0032;
        elsif axi = axi_sjr_tmp_state_0002 then
          tmp_0010 <= tmp_0034;
        elsif axi = axi_sjr_tmp_state_0003 then
          tmp_0010 <= tmp_0036;
        else
          tmp_0010 <= '0';
        end if;
      end if;
    end if;
  end process;

  tmp_0011 <= tmp_0026;

  process(clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        tmp_0012 <= (others => '0');
      else
        if axi = axi_sjr_tmp_state_0001 then
          tmp_0012 <= std_logic_vector(BUF_dout_b);
        elsif axi = axi_sjr_tmp_state_0002 then
          tmp_0012 <= std_logic_vector(BUF_dout_b);
        elsif axi = axi_sjr_tmp_state_0003 then
          tmp_0012 <= std_logic_vector(BUF_dout_b);
        end if;
      end if;
    end if;
  end process;

  tmp_0013 <= sjr_generate_U_0009_dout;

  tmp_0014 <= sjr_generate_U_0009_empty;

  tmp_0015 <= sjr_generate_U_0009_empty;

  tmp_0016 <= sjr_generate_U_0009_count;

  process(clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        tmp_0017 <= (others => '0');
      else
        if axi = axi_IDLE then
          tmp_0017 <= burst_size_sig;
        elsif axi = axi_sjr_tmp_state_0001 then
          tmp_0017 <= tmp_0020;
        elsif axi = axi_sjr_tmp_state_0002 then
          tmp_0017 <= tmp_0020;
        elsif axi = axi_sjr_tmp_state_0003 then
          tmp_0017 <= tmp_0020;
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        tmp_0021 <= (others => '0');
      else
        if axi = axi_IDLE then
          tmp_0021 <= signed(burst_size_sig);
        elsif axi = axi_sjr_tmp_state_0003 then
          tmp_0021 <= tmp_0054;
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        tmp_0024 <= '0';
      else
        if axi = axi_sjr_tmp_state_0003 then
          tmp_0024 <= '1';
        else
          tmp_0024 <= '0';
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        tmp_0058 <= (others => '0');
      else
        if axi = axi_IDLE then
          tmp_0058 <= signed(burst_size_sig);
        elsif axi = axi_sjr_tmp_state_0006 then
          tmp_0058 <= tmp_0066;
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        tmp_0059 <= (others => '0');
      else
        if axi = axi_IDLE then
          tmp_0059 <= (others => '0');
        elsif axi = axi_sjr_tmp_state_0006 then
          tmp_0059 <= tmp_0067;
        end if;
      end if;
    end if;
  end process;


  inst_BUF : dualportram
  generic map(
    WIDTH => 128,
    DEPTH => 10,
    WORDS => 1024
  )
  port map(
    clk => clk,
    reset => reset,
    length => BUF_length,
    address => BUF_address,
    din => BUF_din,
    dout => BUF_dout,
    we => BUF_we,
    oe => BUF_oe,
    address_b => BUF_address_b,
    din_b => BUF_din_b,
    dout_b => BUF_dout_b,
    we_b => BUF_we_b,
    oe_b => BUF_oe_b
  );

  inst_sjr_generate_U_0009 : simple_fifo_128
  port map(
    clk => clk,
    reset => reset,
    we => sjr_generate_U_0009_we,
    din => sjr_generate_U_0009_din,
    re => sjr_generate_U_0009_re,
    dout => sjr_generate_U_0009_dout,
    empty => sjr_generate_U_0009_empty,
    full => sjr_generate_U_0009_full,
    count => sjr_generate_U_0009_count
  );


end RTL;
